<?php

declare(strict_types=1);

namespace Drupal\paywall_rate_limit\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Flood\FloodInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\role_paywall\RolePaywallManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a paywall rate limit information block.
 *
 * @Block(
 *   id = "paywall_rate_limit_paywall_rate_limit_information",
 *   admin_label = @Translation("Paywall rate limit information"),
 *   category = @Translation("Paywall"),
 * )
 */
final class PaywallRateLimitInformationBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs the plugin instance.
   */
  public function __construct(
        array $configuration,
        $plugin_id,
        $plugin_definition,
        private readonly FloodInterface $flood,
        private readonly RouteMatchInterface $routeMatch,
        private readonly RequestStack $requestStack,
        private readonly DateFormatterInterface $dateFormatter,
        private readonly Connection $connection,
        private readonly ConfigFactoryInterface $configFactory,
        private readonly RolePaywallManager $rolePaywallManager
    ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'custom_text' => [
        'value' => $this->t('Your visits to these content types are subjected to a rate limit!'),
        'format' => 'basic_html',
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
          $configuration,
          $plugin_id,
          $plugin_definition,
          $container->get('flood'),
          $container->get('current_route_match'),
          $container->get('request_stack'),
          $container->get('date.formatter'),
          $container->get('database'),
          $container->get('config.factory'),
          $container->get('role_paywall')
      );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state): array {
    // dump($this->configuration['custom_text']);.
    $form['custom_text'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Custom text'),
      '#default_value' => $this->configuration['custom_text']['value'] ?? NULL,
      '#format' => $this->configuration['custom_text']['format'] ?? filter_default_format(),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state): void {
    $this->configuration['custom_text'] = $form_state->getValue('custom_text');
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $node = $this->routeMatch->getParameter('node');
    $node_bundle = $node->bundle();
    $this->rolePaywallManager->setEntityType($node->getEntityTypeId());
    $bundles = $this->rolePaywallManager->getPaywallBundles();

    if (in_array($node_bundle, $bundles)) {
      $session = $this->requestStack->getSession();
      $rate_limit_config = $this->configFactory->get('role_paywall.plugin.rate_limit.node');

      $interval = $rate_limit_config->get('configuration.interval');
      $limit = $rate_limit_config->get('configuration.limit');
      $use_session = $rate_limit_config->get('configuration.use_session_id');

      $build['content'] = [
        '#markup' => $this->t("Not evaluated"),
      ];

      if ($use_session && $session->get('paywall_rate_limit_id')) {
        // $markup = "session {$session->get('paywall_rate_limit_id')}";
        $number = $this->connection
          ->select('flood', 'f')
          ->condition('event', 'paywall_rate_limit.paywall_rate_limit')
          ->condition('identifier', $session->get('paywall_rate_limit_id'))
          ->condition('timestamp', \Drupal::time()->getRequestTime() - $interval, '>')
          ->countQuery()
          ->execute()
          ->fetchField();
        $visits = $this->connection
          ->select('flood', 'f')
          ->fields('f', ['timestamp'])
          ->condition('event', 'paywall_rate_limit.paywall_rate_limit')
          ->condition('identifier', $session->get('paywall_rate_limit_id'))
          ->condition('timestamp', \Drupal::time()->getRequestTime() - $interval, '>')
          ->orderBy('timestamp', 'ASC')
          ->execute()
          ->fetchAll(\PDO::FETCH_ASSOC);
        if (count($visits) > 0) {
          $oldest_event_in_interval = reset($visits)['timestamp'];
        }
        $remain_time = $oldest_event_in_interval + $interval;
        $this->dateFormatter->format($remain_time, 'custom', 'H:i:s');

        $current_interval_since_unix = (\Drupal::time()->getRequestTime() - $interval);
        $current_interval_since = $this->dateFormatter->format($current_interval_since_unix, 'html_time');

        $build['content'] = [
          '#theme' => 'paywall_rate_limit_info',
          '#custom_text' => [
            '#type' => 'processed_text',
            '#text' => $this->configuration['custom_text']['value'],
            '#format' => $this->configuration['custom_text']['format'] ?? filter_default_format(),
          ],
          '#paywalled' => $number < $limit,
          '#visits_value' => $number,
          '#visits_limit' => $limit,
          '#since' => $current_interval_since,
          '#interval' => $interval,
          '#request_time' => \Drupal::time()->getRequestTime(),
          '#time_to_open' => $remain_time,
        ];
      }
    }

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return 0;
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account): AccessResult {
    /*
    // allow if user gets catch per flood or session.
    $session = $this->requestStack->getSession();
    $rate_limit_config = $this->configFactory->get(
    'role_paywall.plugin.rate_limit.node
    ');
    $use_session = $rate_limit_config->get('configuration.use_session_id');
    if ($use_session && $session->get('paywall_rate_limit_id')) {
    return AccessResult::allowedIf(TRUE);
    } else {
    // case for ip case
    $exists_in_flood = $this->flood->isAllowed(
    'paywall_rate_limit.paywall_rate_limit',
    1,
    $this->configuration['interval']);
    return AccessResult::allowedIf($exists_in_flood);
    }
     */

    // Another way to let access the block,
    // such as if the plugin is enabled for current content.
    $node = $this->routeMatch->getParameter('node');
    if (isset($node)) {
      $node_bundle = $node->bundle();
      $this->rolePaywallManager->setEntityType($node->getEntityTypeId());
      $bundles = $this->rolePaywallManager->getPaywallBundles();
      return AccessResult::allowedIf(in_array($node_bundle, $bundles) && in_array('rate_limit', array_keys($this->rolePaywallManager->getPlugins())));
    }
    return AccessResult::allowedIf(FALSE);
  }

}
