<?php

namespace Drupal\paywall_rate_limit\Plugin\RolePaywallAccessRule;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\role_paywall\Plugin\RolePaywallAccessRuleBase;
use Drupal\user\PermissionHandlerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Flood\FloodInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a rate limit access rule for the Role Paywall.
 *
 * @RolePaywallAccessRule(
 *   id = "rate_limit",
 *   label = @Translation("Rate limit"),
 *   description = @Translation("Evaluate access based on a rate limits"),
 * )
 */
class RolePaywallRateLimit extends RolePaywallAccessRuleBase implements ContainerFactoryPluginInterface {

  /**
   * The permission handler.
   *
   * @var \Drupal\user\PermissionHandlerInterface
   */
  protected $permissionHandler;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The flood control mechanism.
   *
   * @var \Drupal\Core\Flood\FloodInterface
   */
  protected $flood;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs a Permission object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\user\PermissionHandlerInterface $permission_handler
   *   The permission handler.
   * @param \Drupal\Core\Flood\FloodInterface $flood
   *   The flood control mechanism.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, PermissionHandlerInterface $permission_handler, FloodInterface $flood, ModuleHandlerInterface $module_handler, RequestStack $request_stack) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->permissionHandler = $permission_handler;
    $this->flood = $flood;
    $this->moduleHandler = $module_handler;
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
          $configuration,
          $plugin_id,
          $plugin_definition,
          $container->get('user.permissions'),
          $container->get('flood'),
          $container->get('module_handler'),
          $container->get('request_stack')
      );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    // The default configuration will be have the block hidden (0).
    return [
      'interval' => '3600',
      'limit' => '10',
      'use_session_id' => TRUE,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function checkAccess(AccountInterface $account, $entity_type, ContentEntityInterface $entity) {

    // $identifier must be based on session
    if ($this->configuration['use_session_id']) {
      $session = $this->requestStack->getSession();
      if (!$session->get('paywall_rate_limit_id')) {
        $random_id = random_int(0, 999999999);
        $ip = $this->requestStack->getCurrentRequest()->getClientIp();
        $session->set('paywall_rate_limit_id', $random_id . "-" . $ip);
      }
      $session_id = $session->get('paywall_rate_limit_id');
    }
    $allowed = ($this->configuration['use_session_id']) ?
        $this->flood->isAllowed('paywall_rate_limit.paywall_rate_limit', $this->configuration['limit'], $this->configuration['interval'], $session_id) :
        $this->flood->isAllowed('paywall_rate_limit.paywall_rate_limit', $this->configuration['limit'], $this->configuration['interval']);
    if ($allowed) {
      if ($this->configuration['use_session_id']) {
        $this->flood->register('paywall_rate_limit.paywall_rate_limit', $this->configuration['interval'], $session_id);
      }
      else {
        $this->flood->register('paywall_rate_limit.paywall_rate_limit', $this->configuration['interval']);
      }
    }
    return $allowed;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm($form, FormStateInterface $form_state) {
    parent::buildConfigurationForm($form, $form_state);

    $form['interval'] = [
      '#type' => 'number',
      '#title' => $this->t('Interval'),
      '#required' => TRUE,
      '#default_value' => $this->configuration['interval'],
      '#description' => $this->t('Interval in seconds to let visit limit content items.'),
    ];
    $form['limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Limit'),
      '#required' => TRUE,
      '#default_value' => $this->configuration['limit'],
      '#description' => $this->t('The quantity of visits that will be allowed when in the interval.'),
    ];
    $form['use_session_id'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use session id instead of ip'),
      '#default_value' => $this->configuration['use_session_id'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['limit'] = $form_state->getValue('limit');
    $this->configuration['interval'] = $form_state->getValue('interval');
    $this->configuration['use_session_id'] = $form_state->getValue('use_session_id');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfigurationSummary() {
    if (empty($this->configuration['limit'])) {
      return $this->t('No limit defined');
    }
    else {
      return $this->t(
            'Matching visitors will be allowed to visit @limit times each @interval seconds, based on @identifier.', [
              '@limit' => $this->configuration['limit'],
              '@interval' => $this->configuration['interval'],
              '@identifier' => ($this->configuration['use_session_id']) ? "session id" : "ip",
            ]
            );
    }
  }

}
